 /*
 * Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#pragma once

#include "uphy_dataset.h"
#include "../../internal_packets/internal_packets_layouts.h"

namespace UPHY
{
    class MadBuilder
    {
        private:
            typedef DataSet::variant_collection_t   collection_t;

        private:
            const DataSet &                         m_dataset;
            const std::list<std::string> &          m_dumps;
            const collection_t &                    m_variants;

        private:
            collection_t::const_iterator            m_iterator;

        public:
            inline const DataSet &                  dataset() const     { return m_dataset;     }
            inline const std::list<std::string> &   dumps() const       { return m_dumps;       }

        public:
            inline bool                             empty() const       { return m_iterator == m_variants.end();    }
            inline collection_t::size_type          size() const        { return m_variants.size();                 }
            inline collection_t::size_type          count() const       { return m_variants.end() - m_iterator;     }

        public:
            inline void                             reset()             { m_iterator = m_variants.begin();          }

        public:
            MadBuilder(const DataSet & dataset, const std::list<std::string> & dumps)
                : m_dataset(dataset), m_dumps(dumps),
                  m_variants(m_dataset.variants(m_dumps)), m_iterator(m_variants.begin())
            {
            }

        public:
            uint16_t next(peucg_reg * peucg);
    };
}
